window.Gleam.initSearch([{"doc":"gleamrpc","title":"gleamrpc","content":"# gleamrpc\n\n[![Package Version](https://img.shields.io/hexpm/v/gleamrpc)](https://hex.pm/packages/gleamrpc)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/gleamrpc/)\n\nRemote Procedure Calls in Gleam !\n\nNote that this library isn&#39;t a standalone library, you will need a client and a server implementations.  \nSee [gleamrpc_http_client](https://hexdocs.pm/gleamrpc_http_client/) and [gleamrpc_http_server](https://hexdocs.pm/gleamrpc_http_server/) for HTTP implementations.\n\n```sh\ngleam add gleamrpc@1\n```\n```gleam\nimport convert\nimport gleam/option\nimport gleamrpc\n\npub fn main() {\n  let id_query_convert = convert.object({\n\t  use id &lt;- convert.field(&quot;id&quot;, fn(v) { Ok(v.id) }, convert.string())\n    convert.success(IdQuery(id))\n  })\n  |&gt; convert.map(\n    fn (v: IdQuery) { v.id },\n    fn (v: String) { Ok(IdQuery(v)) },\n    &quot;&quot;\n  )\n  let user_convert = convert.object({\n    use id &lt;- convert.field(&quot;id&quot;, fn(v) { Ok(v.id) }, convert.string())\n    use name &lt;- convert.field(&quot;name&quot;, fn(v) { Ok(v.name) }, convert.string())\n    use age &lt;- convert.field(&quot;age&quot;, fn(v) { Ok(v.age) }, convert.int())\n    convert.success(User(id, name, age))\n  })\n\n  let users_router = gleamrpc.Router(&quot;users&quot;, option.None) // /api/gleamRPC/users\n  let get_user = gleamrpc.query(&quot;get_user&quot;, option.Some(users_router))\n    |&gt; gleamrpc.params(id_query_convert) // /api/gleamRPC/users/get_user?id=&lt;ID&gt;\n    |&gt; gleamrpc.returns(user_convert)\n\n  // user is of type Result(User, gleamrpc.GleamRPCError)\n  use user &lt;- get_user\n    |&gt; gleamrpc.with_client(client) // client is provided by the implementation you choose\n    |&gt; gleamrpc.call(&quot;1&quot;)\n\n  // Do something with your user\n}\n```\n\nFurther documentation can be found at &lt;https://hexdocs.pm/gleamrpc&gt;.\n\n## Development\n\n```sh\ngleam run   # Run the project\ngleam test  # Run the tests\n```\n","url":"index.html"},{"doc":"gleamrpc","title":"GleamRPCError","content":"pub type GleamRPCError(error) {\n  GleamRPCError(error: error)\n}\n Error wrapper for GleamRPC\nGleamRPCError(error: error)\n\n\nSynonyms:\ngleamrpc.GleamRPCError\ngleamrpc GleamRPCError","url":"gleamrpc.html#GleamRPCError"},{"doc":"gleamrpc","title":"GleamRPCServerError","content":"pub type GleamRPCServerError(error) {\n  WrongProcedure\n  ProcedureExecError(error: ProcedureError)\n  GetIdentityError(error)\n  GetParamsError(errors: List(dynamic.DecodeError))\n}\n Errors that can occur of the server-side\nWrongProcedure\n\n\nProcedureExecError(error: ProcedureError)\n\n\nGetIdentityError(error)\n\n\nGetParamsError(errors: List(dynamic.DecodeError))\n\n\nSynonyms:\ngleamrpc.GleamRPCServerError\ngleamrpc GleamRPCServerError","url":"gleamrpc.html#GleamRPCServerError"},{"doc":"gleamrpc","title":"Procedure","content":"pub type Procedure(params, return) {\n  Procedure(\n    name: String,\n    router: option.Option(Router),\n    type_: ProcedureType,\n    params_type: convert.Converter(params),\n    return_type: convert.Converter(return),\n  )\n}\n A procedure is the equivalent of a function, but the execution of this function can be done remotely\nProcedure(\n  name: String,\n  router: option.Option(Router),\n  type_: ProcedureType,\n  params_type: convert.Converter(params),\n  return_type: convert.Converter(return),\n)\n\n\nSynonyms:\ngleamrpc.Procedure\ngleamrpc Procedure","url":"gleamrpc.html#Procedure"},{"doc":"gleamrpc","title":"ProcedureCall","content":"pub type ProcedureCall(params, return, error) {\n  ProcedureCall(\n    procedure: Procedure(params, return),\n    client: ProcedureClient(params, return, error),\n  )\n}\n A ProcedureCall combines a procedure and a procedure client.  \nProcedureCall(\n  procedure: Procedure(params, return),\n  client: ProcedureClient(params, return, error),\n)\n\n\nSynonyms:\ngleamrpc.ProcedureCall\ngleamrpc ProcedureCall","url":"gleamrpc.html#ProcedureCall"},{"doc":"gleamrpc","title":"ProcedureClient","content":"pub type ProcedureClient(params, return, error) {\n  ProcedureClient(\n    call: fn(\n      Procedure(params, return),\n      params,\n      fn(Result(return, GleamRPCError(error))) -&gt; Nil,\n    ) -&gt;\n      Nil,\n  )\n}\n A procedure client is a way to transmit the data of the procedure to execute and get back its data.  \n You should not create procedure clients directly, but rather use libraries such as `gleamrpc_http_client`.\nProcedureClient(\n  call: fn(\n    Procedure(params, return),\n    params,\n    fn(Result(return, GleamRPCError(error))) -&gt; Nil,\n  ) -&gt;\n    Nil,\n)\n\n\nSynonyms:\ngleamrpc.ProcedureClient\ngleamrpc ProcedureClient","url":"gleamrpc.html#ProcedureClient"},{"doc":"gleamrpc","title":"ProcedureError","content":"pub type ProcedureError {\n  ProcedureError(message: String)\n}\n An error that can occur during the execution of the implementation of a procedure\nProcedureError(message: String)\n\n\nSynonyms:\ngleamrpc.ProcedureError\ngleamrpc ProcedureError","url":"gleamrpc.html#ProcedureError"},{"doc":"gleamrpc","title":"ProcedureHandler","content":"pub type ProcedureHandler(context, error) =\n  fn(\n    ProcedureIdentity,\n    fn(ProcedureType, convert.GlitrType) -&gt;\n      Result(convert.GlitrValue, GleamRPCServerError(error)),\n    context,\n  ) -&gt;\n    Result(convert.GlitrValue, GleamRPCServerError(error))\n A procedure handler is the heart of a procedure server.  \n It is a function that actually does the procedure detection, parameter decoding and implementation call.  \n You don&#39;t have to worry about this type, it is managed by the gleamrpc package.\n\nSynonyms:\ngleamrpc.ProcedureHandler\ngleamrpc ProcedureHandler","url":"gleamrpc.html#ProcedureHandler"},{"doc":"gleamrpc","title":"ProcedureIdentity","content":"pub type ProcedureIdentity {\n  ProcedureIdentity(\n    name: String,\n    router: option.Option(Router),\n    type_: ProcedureType,\n  )\n}\n This is only useful for ProcedureServers\nProcedureIdentity(\n  name: String,\n  router: option.Option(Router),\n  type_: ProcedureType,\n)\n\n\nSynonyms:\ngleamrpc.ProcedureIdentity\ngleamrpc ProcedureIdentity","url":"gleamrpc.html#ProcedureIdentity"},{"doc":"gleamrpc","title":"ProcedureServer","content":"pub type ProcedureServer(transport_in, transport_out, error) {\n  ProcedureServer(\n    get_identity: fn(transport_in) -&gt;\n      Result(ProcedureIdentity, GleamRPCServerError(error)),\n    get_params: fn(transport_in) -&gt;\n      fn(ProcedureType, convert.GlitrType) -&gt;\n        Result(convert.GlitrValue, GleamRPCServerError(error)),\n    recover_error: fn(GleamRPCServerError(error)) -&gt;\n      transport_out,\n    encode_result: fn(convert.GlitrValue) -&gt; transport_out,\n  )\n}\n A procedure server is a way to receive data and map it to the correct procedure implementation.  \n It also handles parameter decoding, result encoding and error handling\n You should not create procedure servers directly, but rather use libraries such as `gleamrpc_http_server`.\nProcedureServer(\n  get_identity: fn(transport_in) -&gt;\n    Result(ProcedureIdentity, GleamRPCServerError(error)),\n  get_params: fn(transport_in) -&gt;\n    fn(ProcedureType, convert.GlitrType) -&gt;\n      Result(convert.GlitrValue, GleamRPCServerError(error)),\n  recover_error: fn(GleamRPCServerError(error)) -&gt; transport_out,\n  encode_result: fn(convert.GlitrValue) -&gt; transport_out,\n)\n\n\nSynonyms:\ngleamrpc.ProcedureServer\ngleamrpc ProcedureServer","url":"gleamrpc.html#ProcedureServer"},{"doc":"gleamrpc","title":"ProcedureServerInstance","content":"pub type ProcedureServerInstance(\n  transport_in,\n  transport_out,\n  context,\n  error,\n) {\n  ProcedureServerInstance(\n    server: ProcedureServer(transport_in, transport_out, error),\n    handler: ProcedureHandler(context, error),\n    context_factory: fn(transport_in) -&gt; context,\n  )\n}\n A ProcedureServerInstance combines a procedure server and handler.  \n It also manages context creation and procedure registration.\nProcedureServerInstance(\n  server: ProcedureServer(transport_in, transport_out, error),\n  handler: ProcedureHandler(context, error),\n  context_factory: fn(transport_in) -&gt; context,\n)\n\n\nSynonyms:\ngleamrpc.ProcedureServerInstance\ngleamrpc ProcedureServerInstance","url":"gleamrpc.html#ProcedureServerInstance"},{"doc":"gleamrpc","title":"ProcedureType","content":"pub type ProcedureType {\n  Query\n  Mutation\n}\n\nQuery\n A query is a procedure that does not alter data, only retrieves it\n\nMutation\n A mutation is a procedure that alters data (think create, update, delete)\n\nSynonyms:\ngleamrpc.ProcedureType\ngleamrpc ProcedureType","url":"gleamrpc.html#ProcedureType"},{"doc":"gleamrpc","title":"Router","content":"pub type Router {\n  Router(name: String, parent: option.Option(Router))\n}\n Routers are a way to organize procedures\nRouter(name: String, parent: option.Option(Router))\n\n\nSynonyms:\ngleamrpc.Router\ngleamrpc Router","url":"gleamrpc.html#Router"},{"doc":"gleamrpc","title":"call","content":"pub fn call(\n  procedure_call: ProcedureCall(a, b, c),\n  params: a,\n  callback: fn(Result(b, GleamRPCError(c))) -&gt; Nil,\n) -&gt; Nil\n Execute the procedure call with the provided parameters.\n It should be used with the &#39;use&#39; syntax.\n \n Example:\n \n \n use data &lt;- my_procedure\n   |&gt; gleamrpc.with_client(my_client())\n   |&gt; gleamrpc.call(my_params)\n \n // do something with data\n  \nSynonyms:\ngleamrpc.call\ngleamrpc call","url":"gleamrpc.html#call"},{"doc":"gleamrpc","title":"mutation","content":"pub fn mutation(\n  name: String,\n  router: Option(Router),\n) -&gt; Procedure(Nil, Nil)\n Create a Mutation procedure\nSynonyms:\ngleamrpc.mutation\ngleamrpc mutation","url":"gleamrpc.html#mutation"},{"doc":"gleamrpc","title":"params","content":"pub fn params(\n  procedure: Procedure(a, b),\n  params_converter: Converter(a),\n) -&gt; Procedure(a, b)\n Set the parameter type of the provided procedure\nSynonyms:\ngleamrpc.params\ngleamrpc params","url":"gleamrpc.html#params"},{"doc":"gleamrpc","title":"query","content":"pub fn query(\n  name: String,\n  router: Option(Router),\n) -&gt; Procedure(Nil, Nil)\n Create a Query procedure\nSynonyms:\ngleamrpc.query\ngleamrpc query","url":"gleamrpc.html#query"},{"doc":"gleamrpc","title":"returns","content":"pub fn returns(\n  procedure: Procedure(a, b),\n  return_converter: Converter(b),\n) -&gt; Procedure(a, b)\n Set the return type of the provided procedure\nSynonyms:\ngleamrpc.returns\ngleamrpc returns","url":"gleamrpc.html#returns"},{"doc":"gleamrpc","title":"serve","content":"pub fn serve(\n  server: ProcedureServerInstance(a, b, c, d),\n) -&gt; fn(a) -&gt; b\n Convert a server instance to a simple function \n \n Example : \n \n \n gleamrpc.with_server(http_server())\n |&gt; gleamrpc.with_implementation(my_procedure, implementation)\n |&gt; gleamrpc.serve\n |&gt; mist.new\n |&gt; mist.start_http\n \nSynonyms:\ngleamrpc.serve\ngleamrpc serve","url":"gleamrpc.html#serve"},{"doc":"gleamrpc","title":"with_client","content":"pub fn with_client(\n  procedure: Procedure(a, b),\n  client: ProcedureClient(a, b, c),\n) -&gt; ProcedureCall(a, b, c)\n Specify the client to use to call the provided procedure\nSynonyms:\ngleamrpc.with_client\ngleamrpc with_client","url":"gleamrpc.html#with_client"},{"doc":"gleamrpc","title":"with_context","content":"pub fn with_context(\n  server: ProcedureServerInstance(a, b, c, d),\n  context_factory: fn(a) -&gt; c,\n) -&gt; ProcedureServerInstance(a, b, c, d)\n Set the procedure server instance&#39;s context factory function\nSynonyms:\ngleamrpc.with_context\ngleamrpc with_context","url":"gleamrpc.html#with_context"},{"doc":"gleamrpc","title":"with_implementation","content":"pub fn with_implementation(\n  server: ProcedureServerInstance(a, b, c, d),\n  procedure: Procedure(e, f),\n  implementation: fn(e, c) -&gt; Result(f, ProcedureError),\n) -&gt; ProcedureServerInstance(a, b, c, d)\n Register a procedure&#39;s implementation in the provided server instance\nSynonyms:\ngleamrpc.with_implementation\ngleamrpc with_implementation","url":"gleamrpc.html#with_implementation"},{"doc":"gleamrpc","title":"with_server","content":"pub fn with_server(\n  server: ProcedureServer(a, b, c),\n) -&gt; ProcedureServerInstance(a, b, a, c)\n Create a procedure server instance for a server\nSynonyms:\ngleamrpc.with_server\ngleamrpc with_server","url":"gleamrpc.html#with_server"},{"doc":"gleamrpc","title":"gleamrpc","content":"","url":"gleamrpc.html"}]);